# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..core.serialization import convert_and_respect_annotation_metadata
from ..errors.bad_request_error import BadRequestError
from ..errors.conflict_error import ConflictError
from ..errors.forbidden_error import ForbiddenError
from ..errors.not_found_error import NotFoundError
from ..types.error import Error
from ..types.tool_parameter import ToolParameter
from .types.create_tool_request_execution_mode import CreateToolRequestExecutionMode
from .types.create_tool_request_type import CreateToolRequestType
from .types.tools_create_response import ToolsCreateResponse
from .types.tools_delete_response import ToolsDeleteResponse
from .types.tools_get_response import ToolsGetResponse
from .types.tools_list_response import ToolsListResponse
from .types.tools_update_response import ToolsUpdateResponse
from .types.update_tool_request_execution_mode import UpdateToolRequestExecutionMode
from .types.update_tool_request_type import UpdateToolRequestType

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawToolsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self, *, project: typing.Optional[str] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[ToolsListResponse]:
        """
        Returns all custom tools for the organization.

        Parameters
        ----------
        project : typing.Optional[str]
            The name of the project to list tools for.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ToolsListResponse]
            Success response
        """
        _response = self._client_wrapper.httpx_client.request(
            "tools",
            base_url=self._client_wrapper.get_environment().base,
            method="GET",
            params={
                "project": project,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ToolsListResponse,
                    parse_obj_as(
                        type_=ToolsListResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create(
        self,
        *,
        name: str,
        description: str,
        type: CreateToolRequestType,
        execution_mode: CreateToolRequestExecutionMode,
        project: typing.Optional[str] = None,
        parameters: typing.Optional[typing.Sequence[ToolParameter]] = OMIT,
        endpoint_method: typing.Optional[typing.Literal["POST"]] = OMIT,
        endpoint_url: typing.Optional[str] = OMIT,
        endpoint_headers: typing.Optional[typing.Dict[str, str]] = OMIT,
        endpoint_timeout_ms: typing.Optional[int] = OMIT,
        tool_call_output_timeout_ms: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[ToolsCreateResponse]:
        """
        Creates a new tool in a project.

        Parameters
        ----------
        name : str
            The name of the tool. Must be snake_case and unique within the organization.

        description : str
            A description of what the tool does.

        type : CreateToolRequestType
            The type of tool.

        execution_mode : CreateToolRequestExecutionMode
            Mode of operation.

        project : typing.Optional[str]
            The name of the project to create the tool in.

        parameters : typing.Optional[typing.Sequence[ToolParameter]]
            Array of parameter definitions.

        endpoint_method : typing.Optional[typing.Literal["POST"]]
            Required for webhook tools.

        endpoint_url : typing.Optional[str]
            Required for webhook tools.

        endpoint_headers : typing.Optional[typing.Dict[str, str]]
            Optional headers for webhook tools.

        endpoint_timeout_ms : typing.Optional[int]
            Timeout for webhook tools.

        tool_call_output_timeout_ms : typing.Optional[int]
            Timeout for WebSocket tool responses.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ToolsCreateResponse]
            Success response
        """
        _response = self._client_wrapper.httpx_client.request(
            "tools",
            base_url=self._client_wrapper.get_environment().base,
            method="POST",
            params={
                "project": project,
            },
            json={
                "name": name,
                "description": description,
                "type": type,
                "execution_mode": execution_mode,
                "parameters": convert_and_respect_annotation_metadata(
                    object_=parameters, annotation=typing.Sequence[ToolParameter], direction="write"
                ),
                "endpoint_method": endpoint_method,
                "endpoint_url": endpoint_url,
                "endpoint_headers": endpoint_headers,
                "endpoint_timeout_ms": endpoint_timeout_ms,
                "tool_call_output_timeout_ms": tool_call_output_timeout_ms,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ToolsCreateResponse,
                    parse_obj_as(
                        type_=ToolsCreateResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get(
        self,
        name_or_id: str,
        *,
        project: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[ToolsGetResponse]:
        """
        Returns a tool by name or ID.

        Parameters
        ----------
        name_or_id : str
            The name or the ID of the tool to get.

        project : typing.Optional[str]
            The name of the project containing the tool. Only used when `nameOrId` is a name.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ToolsGetResponse]
            Success response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"tools/{jsonable_encoder(name_or_id)}",
            base_url=self._client_wrapper.get_environment().base,
            method="GET",
            params={
                "project": project,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ToolsGetResponse,
                    parse_obj_as(
                        type_=ToolsGetResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete(
        self,
        name_or_id: str,
        *,
        project: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[ToolsDeleteResponse]:
        """
        Deletes a tool by name or ID.

        Parameters
        ----------
        name_or_id : str
            The name or the ID of the tool to delete.

        project : typing.Optional[str]
            The name of the project containing the tool. Only used when `nameOrId` is a name.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ToolsDeleteResponse]
            Success response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"tools/{jsonable_encoder(name_or_id)}",
            base_url=self._client_wrapper.get_environment().base,
            method="DELETE",
            params={
                "project": project,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ToolsDeleteResponse,
                    parse_obj_as(
                        type_=ToolsDeleteResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update(
        self,
        name_or_id: str,
        *,
        project: typing.Optional[str] = None,
        name: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        type: typing.Optional[UpdateToolRequestType] = OMIT,
        execution_mode: typing.Optional[UpdateToolRequestExecutionMode] = OMIT,
        parameters: typing.Optional[typing.Sequence[ToolParameter]] = OMIT,
        endpoint_method: typing.Optional[typing.Literal["POST"]] = OMIT,
        endpoint_url: typing.Optional[str] = OMIT,
        endpoint_headers: typing.Optional[typing.Dict[str, str]] = OMIT,
        endpoint_timeout_ms: typing.Optional[int] = OMIT,
        tool_call_output_timeout_ms: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[ToolsUpdateResponse]:
        """
        Updates a tool by name or ID.

        Parameters
        ----------
        name_or_id : str
            The name or the ID of the tool to update.

        project : typing.Optional[str]
            The name of the project containing the tool. Only used when `nameOrId` is a name.

        name : typing.Optional[str]
            The name of the tool. Must be snake_case and unique within the organization.

        description : typing.Optional[str]
            A description of what the tool does.

        type : typing.Optional[UpdateToolRequestType]
            The type of tool.

        execution_mode : typing.Optional[UpdateToolRequestExecutionMode]
            Mode of operation.

        parameters : typing.Optional[typing.Sequence[ToolParameter]]
            Array of parameter definitions.

        endpoint_method : typing.Optional[typing.Literal["POST"]]

        endpoint_url : typing.Optional[str]

        endpoint_headers : typing.Optional[typing.Dict[str, str]]

        endpoint_timeout_ms : typing.Optional[int]

        tool_call_output_timeout_ms : typing.Optional[int]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ToolsUpdateResponse]
            Success response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"tools/{jsonable_encoder(name_or_id)}",
            base_url=self._client_wrapper.get_environment().base,
            method="PATCH",
            params={
                "project": project,
            },
            json={
                "name": name,
                "description": description,
                "type": type,
                "execution_mode": execution_mode,
                "parameters": convert_and_respect_annotation_metadata(
                    object_=parameters, annotation=typing.Sequence[ToolParameter], direction="write"
                ),
                "endpoint_method": endpoint_method,
                "endpoint_url": endpoint_url,
                "endpoint_headers": endpoint_headers,
                "endpoint_timeout_ms": endpoint_timeout_ms,
                "tool_call_output_timeout_ms": tool_call_output_timeout_ms,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ToolsUpdateResponse,
                    parse_obj_as(
                        type_=ToolsUpdateResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawToolsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self, *, project: typing.Optional[str] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[ToolsListResponse]:
        """
        Returns all custom tools for the organization.

        Parameters
        ----------
        project : typing.Optional[str]
            The name of the project to list tools for.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ToolsListResponse]
            Success response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "tools",
            base_url=self._client_wrapper.get_environment().base,
            method="GET",
            params={
                "project": project,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ToolsListResponse,
                    parse_obj_as(
                        type_=ToolsListResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create(
        self,
        *,
        name: str,
        description: str,
        type: CreateToolRequestType,
        execution_mode: CreateToolRequestExecutionMode,
        project: typing.Optional[str] = None,
        parameters: typing.Optional[typing.Sequence[ToolParameter]] = OMIT,
        endpoint_method: typing.Optional[typing.Literal["POST"]] = OMIT,
        endpoint_url: typing.Optional[str] = OMIT,
        endpoint_headers: typing.Optional[typing.Dict[str, str]] = OMIT,
        endpoint_timeout_ms: typing.Optional[int] = OMIT,
        tool_call_output_timeout_ms: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[ToolsCreateResponse]:
        """
        Creates a new tool in a project.

        Parameters
        ----------
        name : str
            The name of the tool. Must be snake_case and unique within the organization.

        description : str
            A description of what the tool does.

        type : CreateToolRequestType
            The type of tool.

        execution_mode : CreateToolRequestExecutionMode
            Mode of operation.

        project : typing.Optional[str]
            The name of the project to create the tool in.

        parameters : typing.Optional[typing.Sequence[ToolParameter]]
            Array of parameter definitions.

        endpoint_method : typing.Optional[typing.Literal["POST"]]
            Required for webhook tools.

        endpoint_url : typing.Optional[str]
            Required for webhook tools.

        endpoint_headers : typing.Optional[typing.Dict[str, str]]
            Optional headers for webhook tools.

        endpoint_timeout_ms : typing.Optional[int]
            Timeout for webhook tools.

        tool_call_output_timeout_ms : typing.Optional[int]
            Timeout for WebSocket tool responses.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ToolsCreateResponse]
            Success response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "tools",
            base_url=self._client_wrapper.get_environment().base,
            method="POST",
            params={
                "project": project,
            },
            json={
                "name": name,
                "description": description,
                "type": type,
                "execution_mode": execution_mode,
                "parameters": convert_and_respect_annotation_metadata(
                    object_=parameters, annotation=typing.Sequence[ToolParameter], direction="write"
                ),
                "endpoint_method": endpoint_method,
                "endpoint_url": endpoint_url,
                "endpoint_headers": endpoint_headers,
                "endpoint_timeout_ms": endpoint_timeout_ms,
                "tool_call_output_timeout_ms": tool_call_output_timeout_ms,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ToolsCreateResponse,
                    parse_obj_as(
                        type_=ToolsCreateResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get(
        self,
        name_or_id: str,
        *,
        project: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[ToolsGetResponse]:
        """
        Returns a tool by name or ID.

        Parameters
        ----------
        name_or_id : str
            The name or the ID of the tool to get.

        project : typing.Optional[str]
            The name of the project containing the tool. Only used when `nameOrId` is a name.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ToolsGetResponse]
            Success response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"tools/{jsonable_encoder(name_or_id)}",
            base_url=self._client_wrapper.get_environment().base,
            method="GET",
            params={
                "project": project,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ToolsGetResponse,
                    parse_obj_as(
                        type_=ToolsGetResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete(
        self,
        name_or_id: str,
        *,
        project: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[ToolsDeleteResponse]:
        """
        Deletes a tool by name or ID.

        Parameters
        ----------
        name_or_id : str
            The name or the ID of the tool to delete.

        project : typing.Optional[str]
            The name of the project containing the tool. Only used when `nameOrId` is a name.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ToolsDeleteResponse]
            Success response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"tools/{jsonable_encoder(name_or_id)}",
            base_url=self._client_wrapper.get_environment().base,
            method="DELETE",
            params={
                "project": project,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ToolsDeleteResponse,
                    parse_obj_as(
                        type_=ToolsDeleteResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update(
        self,
        name_or_id: str,
        *,
        project: typing.Optional[str] = None,
        name: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        type: typing.Optional[UpdateToolRequestType] = OMIT,
        execution_mode: typing.Optional[UpdateToolRequestExecutionMode] = OMIT,
        parameters: typing.Optional[typing.Sequence[ToolParameter]] = OMIT,
        endpoint_method: typing.Optional[typing.Literal["POST"]] = OMIT,
        endpoint_url: typing.Optional[str] = OMIT,
        endpoint_headers: typing.Optional[typing.Dict[str, str]] = OMIT,
        endpoint_timeout_ms: typing.Optional[int] = OMIT,
        tool_call_output_timeout_ms: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[ToolsUpdateResponse]:
        """
        Updates a tool by name or ID.

        Parameters
        ----------
        name_or_id : str
            The name or the ID of the tool to update.

        project : typing.Optional[str]
            The name of the project containing the tool. Only used when `nameOrId` is a name.

        name : typing.Optional[str]
            The name of the tool. Must be snake_case and unique within the organization.

        description : typing.Optional[str]
            A description of what the tool does.

        type : typing.Optional[UpdateToolRequestType]
            The type of tool.

        execution_mode : typing.Optional[UpdateToolRequestExecutionMode]
            Mode of operation.

        parameters : typing.Optional[typing.Sequence[ToolParameter]]
            Array of parameter definitions.

        endpoint_method : typing.Optional[typing.Literal["POST"]]

        endpoint_url : typing.Optional[str]

        endpoint_headers : typing.Optional[typing.Dict[str, str]]

        endpoint_timeout_ms : typing.Optional[int]

        tool_call_output_timeout_ms : typing.Optional[int]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ToolsUpdateResponse]
            Success response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"tools/{jsonable_encoder(name_or_id)}",
            base_url=self._client_wrapper.get_environment().base,
            method="PATCH",
            params={
                "project": project,
            },
            json={
                "name": name,
                "description": description,
                "type": type,
                "execution_mode": execution_mode,
                "parameters": convert_and_respect_annotation_metadata(
                    object_=parameters, annotation=typing.Sequence[ToolParameter], direction="write"
                ),
                "endpoint_method": endpoint_method,
                "endpoint_url": endpoint_url,
                "endpoint_headers": endpoint_headers,
                "endpoint_timeout_ms": endpoint_timeout_ms,
                "tool_call_output_timeout_ms": tool_call_output_timeout_ms,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ToolsUpdateResponse,
                    parse_obj_as(
                        type_=ToolsUpdateResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .extraction_schemas_create_response import ExtractionSchemasCreateResponse
from .extraction_schemas_delete_response import ExtractionSchemasDeleteResponse
from .extraction_schemas_get_response import ExtractionSchemasGetResponse
from .extraction_schemas_list_response import ExtractionSchemasListResponse
from .extraction_schemas_update_response import ExtractionSchemasUpdateResponse

__all__ = [
    "ExtractionSchemasCreateResponse",
    "ExtractionSchemasDeleteResponse",
    "ExtractionSchemasGetResponse",
    "ExtractionSchemasListResponse",
    "ExtractionSchemasUpdateResponse",
]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .types import (
    Agent,
    AgentAudioFormat,
    AgentConfigurationEndpoint,
    AgentProject,
    AgentTemplateVariablesValue,
    AgentToolsItem,
    AssistantChoseNotToRespondPayload,
    AssistantEndedConversationPayload,
    AudioChunkPayload,
    AudioChunkResponsePayload,
    AudioFinishedPayload,
    ConfigPayload,
    ConfigPayloadInputFormat,
    ConfigPayloadOutputFormat,
    Conversation,
    ConversationAnalysis,
    ConversationCreatedPayload,
    ConversationEval,
    ConversationEvalPrompt,
    ConversationEvaluation,
    ConversationEvaluationResult,
    ConversationEvaluationResultPrompt,
    ConversationEvaluationResultResult,
    ConversationExtraction,
    ConversationExtractionSchema,
    ConversationItem,
    ConversationItemRole,
    CreateAgentRequest,
    CreateAgentRequestAudioFormat,
    CreateAgentRequestConfigurationEndpoint,
    CreateAgentRequestTemplateVariablesValue,
    CreateAgentRequestToolsItem,
    DtmfPayload,
    Error,
    ErrorError,
    ErrorPayload,
    ErrorPayloadError,
    ExtractionField,
    ExtractionFieldType,
    ExtractionSchema,
    InputCancelledPayload,
    InputTextPayload,
    InterruptedResponsePayload,
    IsUserSpeakingPayload,
    OutboundCallConfig,
    OutboundCallConfigToolsItem,
    Project,
    ProjectDefaultAgent,
    ReadyToStartConversationPayload,
    SetExternalIdPayload,
    SetTwilioCallSidPayload,
    Task,
    Tool,
    ToolCallInterruptedPayload,
    ToolCallOutputPayload,
    ToolCallOutputProcessedPayload,
    ToolCallOutputProcessedPayloadTool,
    ToolCallPayload,
    ToolExecutionMode,
    ToolParameter,
    ToolParameterItemType,
    ToolParameterType,
    ToolProject,
    ToolType,
    UpdateSystemPromptPayload,
    UserFinishedSpeakingPayload,
    UserStartedSpeakingPayload,
    Voice,
)
from .errors import BadRequestError, ConflictError, ForbiddenError, GatewayTimeoutError, NotFoundError
from . import agents, conversations, extraction_schemas, projects, sts, tools, voices
from .agents import (
    AgentsCreateResponse,
    AgentsDeleteResponse,
    AgentsGetResponse,
    AgentsListResponse,
    AgentsUpdateResponse,
    AgentsUpsertResponse,
    UpdateAgentRequestAudioFormat,
    UpdateAgentRequestConfigurationEndpoint,
    UpdateAgentRequestTemplateVariablesValue,
    UpdateAgentRequestToolsItem,
)
from .client import AsyncPhonic, Phonic
from .conversations import (
    ConversationsCancelResponse,
    ConversationsExtractDataResponse,
    ConversationsGetAnalysisResponse,
    ConversationsGetResponse,
    ConversationsListEvaluationsResponse,
    ConversationsListExtractionsResponse,
    ConversationsListResponse,
    ConversationsListResponseConversation,
    ConversationsListResponseConversations,
    ConversationsOutboundCallResponse,
)
from .environment import PhonicEnvironment
from .extraction_schemas import (
    ExtractionSchemasCreateResponse,
    ExtractionSchemasDeleteResponse,
    ExtractionSchemasGetResponse,
    ExtractionSchemasListResponse,
    ExtractionSchemasUpdateResponse,
)
from .projects import (
    ProjectsCreateEvalPromptResponse,
    ProjectsCreateResponse,
    ProjectsDeleteResponse,
    ProjectsGetResponse,
    ProjectsListEvalPromptsResponse,
    ProjectsListResponse,
    ProjectsUpdateResponse,
)
from .tools import (
    CreateToolRequestExecutionMode,
    CreateToolRequestType,
    ToolsCreateResponse,
    ToolsDeleteResponse,
    ToolsGetResponse,
    ToolsListResponse,
    ToolsUpdateResponse,
    UpdateToolRequestExecutionMode,
    UpdateToolRequestType,
)
from .version import __version__
from .voices import VoicesGetResponse, VoicesListResponse

__all__ = [
    "Agent",
    "AgentAudioFormat",
    "AgentConfigurationEndpoint",
    "AgentProject",
    "AgentTemplateVariablesValue",
    "AgentToolsItem",
    "AgentsCreateResponse",
    "AgentsDeleteResponse",
    "AgentsGetResponse",
    "AgentsListResponse",
    "AgentsUpdateResponse",
    "AgentsUpsertResponse",
    "AssistantChoseNotToRespondPayload",
    "AssistantEndedConversationPayload",
    "AsyncPhonic",
    "AudioChunkPayload",
    "AudioChunkResponsePayload",
    "AudioFinishedPayload",
    "BadRequestError",
    "ConfigPayload",
    "ConfigPayloadInputFormat",
    "ConfigPayloadOutputFormat",
    "ConflictError",
    "Conversation",
    "ConversationAnalysis",
    "ConversationCreatedPayload",
    "ConversationEval",
    "ConversationEvalPrompt",
    "ConversationEvaluation",
    "ConversationEvaluationResult",
    "ConversationEvaluationResultPrompt",
    "ConversationEvaluationResultResult",
    "ConversationExtraction",
    "ConversationExtractionSchema",
    "ConversationItem",
    "ConversationItemRole",
    "ConversationsCancelResponse",
    "ConversationsExtractDataResponse",
    "ConversationsGetAnalysisResponse",
    "ConversationsGetResponse",
    "ConversationsListEvaluationsResponse",
    "ConversationsListExtractionsResponse",
    "ConversationsListResponse",
    "ConversationsListResponseConversation",
    "ConversationsListResponseConversations",
    "ConversationsOutboundCallResponse",
    "CreateAgentRequest",
    "CreateAgentRequestAudioFormat",
    "CreateAgentRequestConfigurationEndpoint",
    "CreateAgentRequestTemplateVariablesValue",
    "CreateAgentRequestToolsItem",
    "CreateToolRequestExecutionMode",
    "CreateToolRequestType",
    "DtmfPayload",
    "Error",
    "ErrorError",
    "ErrorPayload",
    "ErrorPayloadError",
    "ExtractionField",
    "ExtractionFieldType",
    "ExtractionSchema",
    "ExtractionSchemasCreateResponse",
    "ExtractionSchemasDeleteResponse",
    "ExtractionSchemasGetResponse",
    "ExtractionSchemasListResponse",
    "ExtractionSchemasUpdateResponse",
    "ForbiddenError",
    "GatewayTimeoutError",
    "InputCancelledPayload",
    "InputTextPayload",
    "InterruptedResponsePayload",
    "IsUserSpeakingPayload",
    "NotFoundError",
    "OutboundCallConfig",
    "OutboundCallConfigToolsItem",
    "Phonic",
    "PhonicEnvironment",
    "Project",
    "ProjectDefaultAgent",
    "ProjectsCreateEvalPromptResponse",
    "ProjectsCreateResponse",
    "ProjectsDeleteResponse",
    "ProjectsGetResponse",
    "ProjectsListEvalPromptsResponse",
    "ProjectsListResponse",
    "ProjectsUpdateResponse",
    "ReadyToStartConversationPayload",
    "SetExternalIdPayload",
    "SetTwilioCallSidPayload",
    "Task",
    "Tool",
    "ToolCallInterruptedPayload",
    "ToolCallOutputPayload",
    "ToolCallOutputProcessedPayload",
    "ToolCallOutputProcessedPayloadTool",
    "ToolCallPayload",
    "ToolExecutionMode",
    "ToolParameter",
    "ToolParameterItemType",
    "ToolParameterType",
    "ToolProject",
    "ToolType",
    "ToolsCreateResponse",
    "ToolsDeleteResponse",
    "ToolsGetResponse",
    "ToolsListResponse",
    "ToolsUpdateResponse",
    "UpdateAgentRequestAudioFormat",
    "UpdateAgentRequestConfigurationEndpoint",
    "UpdateAgentRequestTemplateVariablesValue",
    "UpdateAgentRequestToolsItem",
    "UpdateSystemPromptPayload",
    "UpdateToolRequestExecutionMode",
    "UpdateToolRequestType",
    "UserFinishedSpeakingPayload",
    "UserStartedSpeakingPayload",
    "Voice",
    "VoicesGetResponse",
    "VoicesListResponse",
    "__version__",
    "agents",
    "conversations",
    "extraction_schemas",
    "projects",
    "sts",
    "tools",
    "voices",
]



# This file was auto-generated by Fern from our API Definition.

import typing

import httpx
from .agents.client import AgentsClient, AsyncAgentsClient
from .conversations.client import AsyncConversationsClient, ConversationsClient
from .core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .environment import PhonicEnvironment
from .extraction_schemas.client import AsyncExtractionSchemasClient, ExtractionSchemasClient
from .projects.client import AsyncProjectsClient, ProjectsClient
from .sts.client import AsyncStsClient, StsClient
from .tools.client import AsyncToolsClient, ToolsClient
from .voices.client import AsyncVoicesClient, VoicesClient


class Phonic:
    """
    Use this class to access the different functions within the SDK. You can instantiate any number of clients with different configuration that will propagate to these functions.

    Parameters
    ----------
    environment : PhonicEnvironment
        The environment to use for requests from the client. from .environment import PhonicEnvironment



        Defaults to PhonicEnvironment.DEFAULT



    token : typing.Union[str, typing.Callable[[], str]]
    headers : typing.Optional[typing.Dict[str, str]]
        Additional headers to send with every request.

    timeout : typing.Optional[float]
        The timeout to be used, in seconds, for requests. By default the timeout is 60 seconds, unless a custom httpx client is used, in which case this default is not enforced.

    follow_redirects : typing.Optional[bool]
        Whether the default httpx client follows redirects or not, this is irrelevant if a custom httpx client is passed in.

    httpx_client : typing.Optional[httpx.Client]
        The httpx client to use for making requests, a preconfigured client is used by default, however this is useful should you want to pass in any custom httpx configuration.

    Examples
    --------
    from phonic import Phonic

    client = Phonic(
        token="YOUR_TOKEN",
    )
    """

    def __init__(
        self,
        *,
        environment: PhonicEnvironment = PhonicEnvironment.DEFAULT,
        token: typing.Union[str, typing.Callable[[], str]],
        headers: typing.Optional[typing.Dict[str, str]] = None,
        timeout: typing.Optional[float] = None,
        follow_redirects: typing.Optional[bool] = True,
        httpx_client: typing.Optional[httpx.Client] = None,
    ):
        _defaulted_timeout = (
            timeout if timeout is not None else 60 if httpx_client is None else httpx_client.timeout.read
        )
        self._client_wrapper = SyncClientWrapper(
            environment=environment,
            token=token,
            headers=headers,
            httpx_client=httpx_client
            if httpx_client is not None
            else httpx.Client(timeout=_defaulted_timeout, follow_redirects=follow_redirects)
            if follow_redirects is not None
            else httpx.Client(timeout=_defaulted_timeout),
            timeout=_defaulted_timeout,
        )
        self.agents = AgentsClient(client_wrapper=self._client_wrapper)
        self.tools = ToolsClient(client_wrapper=self._client_wrapper)
        self.extraction_schemas = ExtractionSchemasClient(client_wrapper=self._client_wrapper)
        self.voices = VoicesClient(client_wrapper=self._client_wrapper)
        self.conversations = ConversationsClient(client_wrapper=self._client_wrapper)
        self.projects = ProjectsClient(client_wrapper=self._client_wrapper)
        self.sts = StsClient(client_wrapper=self._client_wrapper)


class AsyncPhonic:
    """
    Use this class to access the different functions within the SDK. You can instantiate any number of clients with different configuration that will propagate to these functions.

    Parameters
    ----------
    environment : PhonicEnvironment
        The environment to use for requests from the client. from .environment import PhonicEnvironment



        Defaults to PhonicEnvironment.DEFAULT



    token : typing.Union[str, typing.Callable[[], str]]
    headers : typing.Optional[typing.Dict[str, str]]
        Additional headers to send with every request.

    timeout : typing.Optional[float]
        The timeout to be used, in seconds, for requests. By default the timeout is 60 seconds, unless a custom httpx client is used, in which case this default is not enforced.

    follow_redirects : typing.Optional[bool]
        Whether the default httpx client follows redirects or not, this is irrelevant if a custom httpx client is passed in.

    httpx_client : typing.Optional[httpx.AsyncClient]
        The httpx client to use for making requests, a preconfigured client is used by default, however this is useful should you want to pass in any custom httpx configuration.

    Examples
    --------
    from phonic import AsyncPhonic

    client = AsyncPhonic(
        token="YOUR_TOKEN",
    )
    """

    def __init__(
        self,
        *,
        environment: PhonicEnvironment = PhonicEnvironment.DEFAULT,
        token: typing.Union[str, typing.Callable[[], str]],
        headers: typing.Optional[typing.Dict[str, str]] = None,
        timeout: typing.Optional[float] = None,
        follow_redirects: typing.Optional[bool] = True,
        httpx_client: typing.Optional[httpx.AsyncClient] = None,
    ):
        _defaulted_timeout = (
            timeout if timeout is not None else 60 if httpx_client is None else httpx_client.timeout.read
        )
        self._client_wrapper = AsyncClientWrapper(
            environment=environment,
            token=token,
            headers=headers,
            httpx_client=httpx_client
            if httpx_client is not None
            else httpx.AsyncClient(timeout=_defaulted_timeout, follow_redirects=follow_redirects)
            if follow_redirects is not None
            else httpx.AsyncClient(timeout=_defaulted_timeout),
            timeout=_defaulted_timeout,
        )
        self.agents = AsyncAgentsClient(client_wrapper=self._client_wrapper)
        self.tools = AsyncToolsClient(client_wrapper=self._client_wrapper)
        self.extraction_schemas = AsyncExtractionSchemasClient(client_wrapper=self._client_wrapper)
        self.voices = AsyncVoicesClient(client_wrapper=self._client_wrapper)
        self.conversations = AsyncConversationsClient(client_wrapper=self._client_wrapper)
        self.projects = AsyncProjectsClient(client_wrapper=self._client_wrapper)
        self.sts = AsyncStsClient(client_wrapper=self._client_wrapper)



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .tool_call_output_processed_payload_tool import ToolCallOutputProcessedPayloadTool


class ToolCallOutputProcessedPayload(UniversalBaseModel):
    type: typing.Literal["tool_call_output_processed"] = "tool_call_output_processed"
    tool_call_id: str = pydantic.Field()
    """
    ID of the completed tool call
    """

    tool: ToolCallOutputProcessedPayloadTool
    endpoint_url: typing.Optional[str] = pydantic.Field(default=None)
    """
    Webhook endpoint URL (null for WebSocket tools)
    """

    endpoint_timeout_ms: typing.Optional[float] = pydantic.Field(default=None)
    """
    Webhook timeout in milliseconds (null for WebSocket tools)
    """

    endpoint_called_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    When webhook was called (null for WebSocket tools)
    """

    request_body: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = pydantic.Field(default=None)
    """
    Webhook request body (null for WebSocket tools)
    """

    response_body: typing.Optional[typing.Optional[typing.Any]] = None
    parameters: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = pydantic.Field(default=None)
    """
    WebSocket tool parameters (null for webhook tools)
    """

    output: typing.Optional[typing.Optional[typing.Any]] = None
    response_status_code: typing.Optional[float] = pydantic.Field(default=None)
    """
    Webhook HTTP status code (null for WebSocket tools)
    """

    timed_out: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether the tool call timed out
    """

    error_message: typing.Optional[str] = pydantic.Field(default=None)
    """
    Error message if tool call failed
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
